#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <GL/glut.h>

#include "vector.h"
#include "Obj.h"
#include "Saucer.h"
#include "Wall.h"

struct _Wall {
  double stand_angle;
  Block block;
  Saucer saucer;
};

Wall Wall_Create(Block block, Saucer saucer)
{
  Wall wall;

  wall = malloc(sizeof(*wall));
  memset(wall, 0, sizeof(*wall));

  wall->stand_angle = 90;
  wall->block = block;
  wall->saucer = saucer;

  return wall;
}

Wall Wall_Destroy(Wall wall)
{
  if (wall) free(wall);
  return NULL;
}

int Wall_Proc(Wall wall)
{
  if (Block_GetStatus(wall->block) == BLOCK_STATUS_CLEARING) {
    if (wall->stand_angle == 0) {
      Block_SetStatus(wall->block, BLOCK_STATUS_CLEARED);
    } else {
      wall->stand_angle -= 10;
    }
  }
  return 1;
}

int Wall_MakePolygon(Wall wall, int track)
{
  int track_num, sector_num;
  vector_t p0, p1;
  vector_t points[6];
  Question question;
  GLfloat red[]     = { 0.7, 0.0, 0.0, 0.6 };
  GLfloat green[]   = { 0.0, 0.7, 0.0, 0.6 };
  GLfloat blue[]    = { 0.0, 0.5, 0.7, 0.6 };
  GLfloat yellow[]  = { 0.7, 0.7, 0.0, 0.6 };
  GLfloat magenta[] = { 0.7, 0.0, 0.7, 0.6 };
  GLfloat *color;

  track_num = Saucer_GetTrackNum(wall->saucer);
  sector_num = Saucer_GetSectorNum(wall->saucer);

  switch (Block_GetStatus(wall->block)) {
  default:
  case BLOCK_STATUS_UNCLEAR:  color = blue;    break;
  case BLOCK_STATUS_TRY:      color = magenta; break;
  case BLOCK_STATUS_ATTACK:   color = red;     break;
  case BLOCK_STATUS_CLEARING: color = yellow;  break;
  case BLOCK_STATUS_CLEARED:  color = green;   break;
  }

  glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT , color);
  glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE , color);
  glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, color);

  vector_set(&p0,
	     cos(M_PI / sector_num) * (1.0 / (track_num + 1)) * (track + 2),
	     sin(M_PI / sector_num) * (1.0 / (track_num + 1)) * (track + 2),
	     0.0);
  vector_set(&p1,
	     cos(M_PI / sector_num) * (1.0 / (track_num + 1)) * (track + 1),
	     sin(M_PI / sector_num) * (1.0 / (track_num + 1)) * (track + 1),
	     0.0);

  vector_set(&points[0], p1.x - p0.x, -p1.y, 0.0);
  vector_set(&points[1],         0.0, -p0.y, 0.0);
  vector_set(&points[2],         0.0, -p1.y, 0.0);
  vector_set(&points[3],         0.0,  p1.y, 0.0);
  vector_set(&points[4],         0.0,  p0.y, 0.0);
  vector_set(&points[5], p1.x - p0.x,  p1.y, 0.0);

  glRotated(wall->stand_angle, 0.0, 1.0, 0.0);

  question = Block_GetQuestion(wall->block);

  if (question) {
    glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA,
		 Question_GetImageWidth(question),
		 Question_GetImageHeight(question),
		 0, GL_RGBA, GL_UNSIGNED_BYTE, Question_GetImage(question));
    glEnable(GL_TEXTURE_2D);
  }

  glBegin(GL_POLYGON);
  glNormal3d(0.0, 0.0, 1.0);

  if (question) glTexCoord3d(0.0, 0.0, 0.0);
  glVertex3d(points[0].x, points[0].y, points[0].z);
  if (question) glTexCoord3d(0.0, 1.0, 0.0);
  glVertex3d(points[2].x, points[2].y, points[2].z);
  if (question) glTexCoord3d(1.0, 1.0, 0.0);
  glVertex3d(points[3].x, points[3].y, points[3].z);
  if (question) glTexCoord3d(1.0, 0.0, 0.0);
  glVertex3d(points[5].x, points[5].y, points[5].z);

  glEnd();

  if (question) {
    glDisable(GL_TEXTURE_2D);
  }

  glBegin(GL_POLYGON);
  glNormal3d(0.0, 0.0, 1.0);
  glVertex3d(points[0].x, points[0].y, points[0].z);
  glVertex3d(points[1].x, points[1].y, points[1].z);
  glVertex3d(points[2].x, points[2].y, points[2].z);
  glEnd();

  glBegin(GL_POLYGON);
  glNormal3d(0.0, 0.0, 1.0);
  glVertex3d(points[3].x, points[3].y, points[3].z);
  glVertex3d(points[4].x, points[4].y, points[4].z);
  glVertex3d(points[5].x, points[5].y, points[5].z);
  glEnd();

  return 0;
}
